﻿# DBBuilder

## ISCript language (ISC)

**ISC** è un semplice [DSL](https://en.wikipedia.org/wiki/Domain-specific_language) progettato per semplificare la *creazione*, *manutenzione* e *porting* di database SQL. Il linguaggio prevede una serie di costrutti che riprendono una sintassi vicina a quella di SQL estendendone però le funzionalità.

### Commenti

Il linguaggio permette l'uso di commenti a singola linea sia con doppio slash `//` che con cancelletto `#`. I commenti a riga multipla invece devono iniziare con `/*` e finire con `*/`.

### Stringhe

Ci sono due tipi di stringhe: quelle delimitate dalle virgolette `"` e quelle delimitate dalle triple virgolette `"""`. Nel primo caso alcuni caratteri (quelli che iniziano con backslash `\`) vengono interpretati per esempio:

* `\\` diventa il carattere `\`
* `\n` diventa una interruzione di linea (CR+LF)
* `\r` diventa un ritorno carrello (CR)
* `\t` diventa una tabulazione (TAB) 
* `\"` diventa il carattere `"`

Le stringhe delimitate con tre virgolette invece vengono lette così come sono scritte. Per esempio:

```
"""Testo molto lungo
che può anche andare a capo e contenere
anche dei "caratteri speciali" che \non\ verranno
interpretati. """
```

### Connessioni

All'apertura del programma vengono aperte due connessioni `SOURCE` e `TARGET` il database di riferimento delle connessioni viene impostato attraverso il file di configurazione.

I comandi elencati di seguito nella maggior parte dei casi agiscono su una connessione denominata `DEFAULT_CONNECTION` che all'avvio è uguale a `TARGET`. È possibile modificare la connessione di default tramite la modifica di una variabile di sistema (vedi comando `SET`).  
### Comando LOG

```
LOG [ DEBUG | INFO | WARNING | ERROR ] testo
```

Scrive un semplice messaggio di testo. Opzionalmente è possibile indicare il livello di log (debug, info, warning, error).

```sql
LOG "MESSAGGIO DI TEST";
LOG DEBUG "MESSAGGIO DI DEBUG";
LOG INFO "MESSAGGIO INFO";
LOG WARNING "MESSAGGIO WARNING";
LOG ERROR "MESSAGGIO ERRORE";
```

### Comando MAP

```
associazione ::= nome_campo_sorgente AS nome_campo_destinazione

CREATE MAP FOR nome_tabella_destinazione ( associazione [,associazione [,associazione]]
```

Il comando `MAP` permette di associare i nomi dei campi di una tabella di origine a quelli di una tabella di destinazione. In questo modo le successive chiamate al comando `COPY` terranno conto delle informazioni presenti nella mappa.

In una mappa non è necessario specificare tutti i campi di una tabella ma i campi non specificati espressamente non verranno utilizzati dai comandi che ne fanno uso (per esempio `COPY`)

```sql
CREATE MAP FOR UTENTI (
    UTE_ID AS ID_UTENTE,
    UTE_NOME AS NOME,
    UTE_PASSWORD AS PASSWORD,
    GRP_ID AS ID_GRUPPO,
    UTE_EMAIL AS EMAIL,
    UTE_DATA_LOGIN AS DATA_LOGIN,
    UTE_DATA_LAST_LOGIN AS DATA_LAST_LOGIN,
    UTE_DATA_CREA AS DATA_CREA,
    UTE_DATA_MODI AS DATA_MODI,
    UTE_DATA_PASSWORD AS DATA_PASSWORD 
);
```

### Comando COPY

```
COPY FROM tabella_sorgente TO tabella_destinazione [ FILTER "condizione"];
```

Il comando copia una tabella dal database `SOURCE` al database `TARGET`. Il processo richiede che la tabella di destinazione sia già stata creata. Se esiste una *mappa* (vedi comando `MAP`) viene usata automaticamente. 

Opzionalmente è possibile indicare un filtro. Il filtro è una stringa che, se presente, verrà aggiunta alla query generata sul database `SOURCE` per ottenere i dati.

```sql
COPY FROM UTENTI TO UTENTI FILTER " UTENTE_ATTIVO = 'S' ";
```

### Comando SYNC SEQUENCE

```
SYNC SEQUENCE nome_sequence WITH tabella.campo;
```

Il comando legge il valore massimo del campo specificato e imposta la sequence indicata col valore trovato.

```sql
SYNC SEQUENCE GEN_UTENTI WITH UTENTI.ID_UTENTE;
```

### Comando ALTER TRIGGER

```
ALTER TRIGGER ALL stato;
```

Il comando permette di abilitare e disabilitare tutti i trigger del database di destinazione. I valori per *stato* sono ACTIVE e INACTIVE. 

```sql
ALTER TRIGGER ALL INACTIVE;
```

### Comando EXECUTE SQL

```
EXECUTE SQL "comando SQL";
```

Questo comando permette di inviare uno script SQL al motore di database. Il testo viene inviato così com'è senza nessuna interpretazione da parte del motore SQL.

```sql
EXECUTE SQL """
  
  CREATE TABLE TEST (
    ID_TEST INTEGER;
    TEXT VARCHAR(10)
  );

""";
```
### Comando EXECUTE QUERY

```
EXECUTE QUERY "query QUERY";
```

Questo comando permette di far eseguire delle query SQL al motore di database. Il testo viene inviato così com'è senza nessuna interpretazione da parte del motore SQL e il risultato viene inviato come log.

```sql
EXECUTE QUERY """
  
  SELECT * 
  FROM TABELLA1
  INNER JOIN TABELLA2 ON (....)
  WHERE ....
  ORDER BY ...

""";
```

### Comando SET
```
SET NOME_VARIABILE valore;
```
Il comando SET imposta una variabile al valore indicato. Il valore più essere una stringa, un numero, un booleano o NULL.

Esisto delle variabili di sistema che possono essere modificate con il comando `SET` o visualizzate con il comando `SHOW`:

| Variable               | Tipo       | RO   | Significato |
| ---                    | ---        | ---- |---          |
| DEFAULT_CONNECTION     | stringa    | No   | Indica il nome della connessione di dafault (SOURCE o TARGET)   |
| SOURCE_DATABASE        | stringa    | Sì   | Visualizza i parametri della connessione SOURCE   |
| TARGET_DATABASE        | stringa    | Sì   | Visualizza i parametri della connessione TARGET   |
| EXE_NAME               | stringa    | Sì   | Visualizza il nome dell'eseguibile   |

```sql
SET DEFAULT_CONNECTION "SOURCE";
```

### Comando SHOW

```
SHOW NOME_VARIABILE;
```
Il comando SHOW permette di visualizzare una variabile utente o una variabile di sistema. Il valore viene visualizzato come stringa usando le impostazioni della macchina per la visualizzazione (per esempio la scelta del punto o della virgola per i numeri decimali).

```sql
SHOW DEFAULT_CONNECTION;
```
