unit DBBuilder.Actions.Batch;

interface

uses
  System.Classes, System.SysUtils, System.IOUtils,
  DBBuilder.Actions.Core, Dynamo.Core.Rtti;

type
  [Alias('BAT')]
  TBatchAction = class(TInterfacedObject, IAction)
  public
    procedure ExecuteFile(const LFileName: string);
  end;

implementation

{ TBatchAction }

uses
  DBBuilder.Core.Config,
  Dynamo.Core.ServiceLocator, WinApi.ShellApi, WinApi.Windows;

procedure ShellExecuteAndWait(const AFileName: string; AParams: string);
var
  exInfo: TShellExecuteInfo;
  PHandler: DWORD;
begin

  FillChar(exInfo, SizeOf(exInfo), 0);
  with exInfo do
  begin
    cbSize := SizeOf(exInfo);
    fMask := SEE_MASK_NOCLOSEPROCESS or SEE_MASK_FLAG_DDEWAIT;
    Wnd := GetActiveWindow();
    exInfo.lpVerb := 'open';
    exInfo.lpParameters := PChar(AParams);
    lpFile := PChar(AFileName);
    nShow := SW_SHOWNORMAL;
  end;
  if not ShellExecuteEx(@exInfo) then
    raise EActionError.Create(SysErrorMessage(GetLastError));

  PHandler := exInfo.hProcess;
  try
    while WaitForSingleObject(exInfo.hProcess, 50) <> WAIT_OBJECT_0 do
      Sleep(100);
  finally
    CloseHandle(PHandler);
  end;
end;

procedure TBatchAction.ExecuteFile(const LFileName: string);
var
  LFilePath: string;
begin
  LFilePath := TPath.Combine(Config.ScriptPath, LFileName);
  ShellExecuteAndWait('cmd', Format('/c "%s"', [LFilePath]));
end;

initialization

  ServiceLocator.RegisterClass(TBatchAction);

end.
