unit DBBuilder.Core.Logging;

interface

uses
  System.Classes, System.SysUtils;

type
  TLogLevel = (llDebug, llInfo, llWarning, llError);

  ILogger = interface
    ['{24A108CD-0B5D-4482-B320-8572C98E529D}']
    procedure Log(ALogLevel: TLogLevel; const AMessage: string);
  end;

  TLogger = class(TObject)
  public
    class procedure Log(ALogLevel: TLogLevel; const AMessage: string); overload; static;
    class procedure Log(ALogLevel: TLogLevel; const AFormat: string; const AArgs: array of const); overload; static;
  end;

procedure RegisterLogger(ALogger: ILogger);

implementation

var
  _Logger: ILogger;

procedure RegisterLogger(ALogger: ILogger);
begin
  _Logger := ALogger;
end;

{ TLogger }

class procedure TLogger.Log(ALogLevel: TLogLevel; const AMessage: string);
begin
  if Assigned(_Logger) then
    _Logger.Log(ALogLevel, AMessage);
end;

class procedure TLogger.Log(ALogLevel: TLogLevel; const AFormat: string;
  const AArgs: array of const);
var
  LMessage: string;
begin
  LMessage := Format(AFormat, AArgs);
  TLogger.Log(ALogLevel, LMessage);
end;

end.
