unit DBBuilder.Core.Scanner;

interface

uses
  System.Classes, System.SysUtils, System.IOUtils, System.Types;

type
  TFileInfo = record
  private
    FFileName: string;
    FApplied: Boolean;
    FScriptName: string;
  public
    property ScriptName: string read FScriptName;
    property FileName: string read FFileName;
    property Applied: Boolean read FApplied;
    constructor Create(const AFileName: string; AApplied: Boolean);
  end;

  TFileInfoList = TArray<TFileInfo>;

  TFileScanner = record
  public
    class function Scan(): TFileInfoList; static;
  end;

implementation

{ TFileScanner }

uses
  DBBuilder.Core.Config,
  DBBuilder.DB.DatabaseManager;

class function TFileScanner.Scan: TFileInfoList;
var
  LFiles: TStringDynArray;
  LFileName: string;
  LIndex: Integer;
  LApplied: Boolean;
begin
  TargetDatabaseManager.ClearCache;
  LFiles := TDirectory.GetFiles(Config.ScriptPath);
  SetLength(Result, Length(LFiles));
  LIndex := 0;
  for LFileName in LFiles do
  begin
    LApplied := TargetDatabaseManager.CheckForScript(ExtractFileName(LFileName));
    Result[LIndex] := TFileInfo.Create(LFileName, LApplied);
    Inc(LIndex);
  end;
end;

{ TFileInfo }

constructor TFileInfo.Create(const AFileName: string; AApplied: Boolean);
begin
  FFileName := AFileName;
  FScriptName := ExtractFileName(AFileName);
  FApplied := AApplied;
end;

end.
