unit DBBuilder.DB.Core;

interface

uses
  System.Classes, System.SysUtils, System.Variants, System.Generics.Collections,
  Data.DB;

type
  TTriggerStatus = (tsInactive, tsActive);

  TDBParam = class
  private
    FName: string;
    FDataType: TFieldType;
    FValue: Variant;
  public
    property Name: string read FName;
    property Value: Variant read FValue write FValue;
    property DataType: TFieldType read FDataType write FDataType;
  end;

  TDBParams = class
  private
    FParams: TObjectDictionary<string,TDBParam>;
  public
    function ToArray: TArray<TDBParam>;
    procedure Add(const AName: string; ADataType: TFieldType; AValue: Variant);
    function GetParamByName(const AName: string): Variant;
    procedure Clear;

    constructor Create;
    destructor Destroy; override;
  end;

implementation

{ TDBParams }

procedure TDBParams.Add(const AName: string; ADataType: TFieldType;
  AValue: Variant);
var
  LDBParam: TDBParam;
begin
  if FParams.TryGetValue(AName, LDBParam) then
  begin
    LDBParam.FValue := AValue;
  end
  else
  begin
    LDBParam := TDBParam.Create;
    try
      LDBParam.FName := AName;
      LDBParam.DataType := ADataType;
      LDBParam.Value := AValue;
      FParams.Add(AName, LDBParam);
    except
      LDBParam.Free;
      raise;
    end;
  end;
end;

procedure TDBParams.Clear;
begin
  FParams.Clear;
end;

constructor TDBParams.Create;
begin
  FParams := TObjectDictionary<string,TDBParam>.Create([doOwnsValues]);
end;

destructor TDBParams.Destroy;
begin
  FParams.Free;
  inherited;
end;

function TDBParams.GetParamByName(const AName: string): Variant;
var
  LDBParam: TDBParam;
begin
  if not FParams.TryGetValue(AName, LDBParam) then
    Exit(Null);

  Result := LDBParam.Value;
end;

function TDBParams.ToArray: TArray<TDBParam>;
var
  LArray: TArray<TPair<string,TDBParam>>;
  LIndex: Integer;
begin
  LArray := FParams.ToArray;
  SetLength(Result, Length(LArray));
  for LIndex := 0 to Length(LArray) - 1 do
    Result[LIndex] := LArray[LIndex].Value;
end;

end.
