unit DBBuilder.DB.TestDatabaseEngine;

interface

uses
  System.Classes, System.SysUtils, Data.DB, DBBuilder.Core.Config,
  DBBuilder.DB.DatabaseEngine, Dynamo.Core.Rtti, System.Variants,
  Datasnap.DBClient, DBBuilder.DB.Core;

type
  [Alias('test')]
  TTestDatabaseEngine = class(TInterfacedObject, IDatabaseEngine)
  private
    FConnect: Boolean;
    procedure LogStatement(const AStatement: string);
  public
    function GetAllTriggers: TArray<string>;
    procedure AlterTriggerStatus(const ATriggerName: string; ATriggerStatus: TTriggerStatus);
    function CheckTable(const ATableName: string): Boolean;
    procedure Connect(ADatabase: TDatabaseInfo);
    function CreateDataSet(const ASQL: string): TDataSet;
    function DateTimeToDBStr(ADateTime: TDateTime): string;
    function DateToDBStr(ADateTime: TDateTime): string;
    procedure Disconnect;
    function QuoteValue(AValue: Variant; AFieldType: TFieldType): string;
    function GetConnect: Boolean;
    procedure Commit;
    procedure Rollback;
    procedure SetSequence(const ASequenceName: string; AValue: Int64);
    procedure ExecuteScript(const AStatement: string);
    procedure ExecuteStatement(const AStatement: string; AParams: TDBParams);
  end;


implementation

{ TTestDatabaseEngine }

uses
  DBBuilder.Core.Logging, Dynamo.Core.ServiceLocator;

procedure TTestDatabaseEngine.AlterTriggerStatus(const ATriggerName: string;
  ATriggerStatus: TTriggerStatus);
const
  cTriggerStatusStr: array [TTriggerStatus] of string = ('INACTIVE', 'ACTIVE');
begin
  TLogger.Log(llInfo, 'TEST_ENGINE: alter trigger "%s" status: "%s"', [ATriggerName, cTriggerStatusStr[ATriggerStatus]]);
end;

function TTestDatabaseEngine.CheckTable(const ATableName: string): Boolean;
begin
  Result := True;
end;

procedure TTestDatabaseEngine.Commit;
begin
  TLogger.Log(llInfo, 'TEST_ENGINE: commit');
end;

procedure TTestDatabaseEngine.Connect(ADatabase: TDatabaseInfo);
begin
  FConnect := True;
  TLogger.Log(llInfo, 'TEST_ENGINE: connect');
end;

function TTestDatabaseEngine.CreateDataSet(const ASQL: string): TDataSet;
var
  LDataSet: TClientDataSet;
begin
  LogStatement(ASQL);
  LDataSet := TClientDataSet.Create(nil);
  try
    LDataSet.FieldDefs.Add('id', ftInteger);
    LDataSet.FieldDefs.Add('test', ftString, 100);
    LDataSet.CreateDataSet;
    LDataSet.LogChanges := False;
    LDataSet.AppendRecord([1, 'Luca']);
    LDataSet.AppendRecord([2, 'Mario']);
    LDataSet.First;
    Result := LDataSet;
  except
    LDataSet.Free;
    raise;
  end;
end;

function TTestDatabaseEngine.DateTimeToDBStr(ADateTime: TDateTime): string;
begin
  Result := '''' + DateTimeToStr(ADateTime) + '''';
end;

function TTestDatabaseEngine.DateToDBStr(ADateTime: TDateTime): string;
begin
  Result := '''' + DateToStr(ADateTime) + '''';
end;

procedure TTestDatabaseEngine.Disconnect;
begin
  FConnect := False;
  TLogger.Log(llInfo, 'TEST_ENGINE: disconnect');
end;

procedure TTestDatabaseEngine.LogStatement(const AStatement: string);
var
  LMessage: string;
begin
  if Length(AStatement) > 100 then
    LMessage := Copy(AStatement, 1, 100) + '...'
  else
    LMessage := AStatement;
  TLogger.Log(llInfo, '"%s"', [LMessage]);
end;

procedure TTestDatabaseEngine.ExecuteScript(const AStatement: string);
begin
  LogStatement('EXECUTE SCRIPT: ' + AStatement);
end;

procedure TTestDatabaseEngine.ExecuteStatement(const AStatement: string;
  AParams: TDBParams);
begin
  LogStatement('EXECUTE STATEMENT: ' + AStatement);
end;

function TTestDatabaseEngine.GetAllTriggers: TArray<string>;
begin
  Result := [];
  Result := Result + ['TRG_TEST_1'];
  Result := Result + ['TRG_TEST_2'];
  Result := Result + ['TRG_TEST_3'];
end;

function TTestDatabaseEngine.GetConnect: Boolean;
begin
  Result := FConnect;
end;

function TTestDatabaseEngine.QuoteValue(AValue: Variant;
  AFieldType: TFieldType): string;
begin
  Result := VarToStr(AValue);
end;

procedure TTestDatabaseEngine.Rollback;
begin
  TLogger.Log(llInfo, 'TEST_ENGINE: rollback');
end;

procedure TTestDatabaseEngine.SetSequence(const ASequenceName: string;
  AValue: Int64);
begin
  TLogger.Log(llInfo, 'TEST_ENGINE: Sequence "%s" impostata a "%d"', [ASequenceName, AValue]);
end;

initialization

  ServiceLocator.RegisterClass(TTestDatabaseEngine);

end.
