unit DBBuilder.UI.MainForm;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants,
  System.Classes, System.IOUtils, System.Types, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.StdCtrls, Vcl.CheckLst,
  System.ImageList, Vcl.ImgList, System.UITypes, Vcl.Imaging.pngimage,
  Vcl.ExtCtrls, DBBuilder.Core.Logging, Vcl.ComCtrls;

type
  TfrmMain = class(TForm, ILogger)
    lstScripts: TCheckListBox;
    btnEseguiTutto: TButton;
    ImageList1: TImageList;
    boxHeader: TPanel;
    btnEseguiScript: TButton;
    memScript: TMemo;
    memLog: TRichEdit;
    imgFiller: TPaintBox;
    procedure btnEseguiScriptClick(Sender: TObject);
    procedure btnEseguiTuttoClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure imgFillerPaint(Sender: TObject);
    procedure lstScriptsClick(Sender: TObject);
  private
    FLogoImage: TPngImage;
    procedure InitListBox;
    procedure ExecuteScript(const FileName: string);
    procedure LoadMaps;
  public
    { ILogger }
    procedure Log(ALogLevel: TLogLevel; const AMessage: string);

    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  end;

var
  frmMain: TfrmMain;

implementation

{$R *.dfm}

uses
  DBBuilder.Core.Scanner, DBBuilder.Actions.Core, DBBuilder.DB.DatabaseManager,
  DBBuilder.Core.Config, DBBuilder.ISCScript.Processor;

procedure TfrmMain.btnEseguiScriptClick(Sender: TObject);
var
  LScriptName: string;
begin
  LoadMaps;
  if lstScripts.ItemIndex < 0 then
    raise Exception.Create('Nessun elemento selezionato');

  if not lstScripts.Checked[lstScripts.ItemIndex] then
  begin
    if MessageDlg('Script gi caricato. Continuare?', mtConfirmation, mbYesNo, 0) <> mrYes then
      Abort;
  end;

  LScriptName := lstScripts.Items[lstScripts.ItemIndex];

  TAction.ExecuteFile(LScriptName);
  TargetDatabaseManager.StoreAction(LScriptName);
  lstScripts.Checked[lstScripts.ItemIndex] := False;
  TargetDatabaseManager.ClearCache;
end;

procedure TfrmMain.btnEseguiTuttoClick(Sender: TObject);
var
  LIndex: Integer;
  LStartTime, LEndTime: TDateTime;
  LCount: Integer;
begin
  LoadMaps;
  LStartTime := Now;
  LCount := 0;
  for LIndex := 0 to lstScripts.Count - 1 do
  begin
    if lstScripts.Checked[LIndex] then
    begin
      ExecuteScript(lstScripts.Items[LIndex]);
      lstScripts.Checked[LIndex] := False;
      inc(LCount);
      //Sleep(1000);
    end;
  end;
  LEndTime := Now;
  TargetDatabaseManager.ClearCache;
  InitListBox;

  TLogger.Log(llInfo, 'Eseguiti %d script in %s secondi', [LCount, TimeToStr(LEndTime - LStartTime)]);
  MessageDlg('Caricamento script terminato', mtInformation, [mbOK], 0);
end;

constructor TfrmMain.Create(AOwner: TComponent);
begin
  inherited;
  RegisterLogger(Self);

  FLogoImage := TPngImage.Create;
  FLogoImage.LoadFromResourceName(HInstance, 'DBBUILDER_LOGO');
end;

destructor TfrmMain.Destroy;
begin
  RegisterLogger(nil);
  FLogoImage.Free;
  inherited;
end;

procedure TfrmMain.ExecuteScript(const FileName: string);
begin
  TAction.ExecuteFile(FileName);
  TargetDatabaseManager.StoreAction(FileName);
end;

procedure TfrmMain.FormCreate(Sender: TObject);
begin
  InitListBox;
end;

procedure TfrmMain.imgFillerPaint(Sender: TObject);
const
  RectHeight = 153;
begin
  if Assigned(FLogoImage) then
  begin
    // Sfondo
    imgFiller.Canvas.Brush.Color := $D49C2D;
    imgFiller.Canvas.FillRect(Rect(0, 0, imgFiller.Width, RectHeight));
    imgFiller.Canvas.Brush.Color := clWhite;
    imgFiller.Canvas.FillRect(Rect(0, RectHeight, imgFiller.Width, imgFiller.Height));

    // Logo
    imgFiller.Canvas.Draw(Width - FLogoImage.Width, 0, FLogoImage);

    // Titolo
    imgFiller.Canvas.Brush.Style := bsClear;
    imgFiller.Canvas.Font.Size := 50;
    imgFiller.Canvas.Font.Color := clDkGray;
    imgFiller.Canvas.TextOut(30, 20, Caption);
    imgFiller.Canvas.Brush.Style := bsClear;
    imgFiller.Canvas.Font.Color := clWhite;
    imgFiller.Canvas.TextOut(32, 22, Caption);

  end;
end;

procedure TfrmMain.InitListBox;
var
  LFileInfo: TFileInfo;
  LIndex: Integer;
begin
  TLogger.Log(llInfo, 'Lettura degli script');
  LIndex := 0;
  lstScripts.Clear;
  for LFileInfo in TFileScanner.Scan() do
  begin
    lstScripts.Items.Add(LFileInfo.ScriptName);
    lstScripts.Checked[LIndex] := not LFileInfo.Applied;
    Inc(LIndex);
  end;
end;

procedure TfrmMain.LoadMaps;
var
  LFiles: TStringDynArray;
  LFileName: string;
begin
  TLogger.Log(llInfo, 'Caricamento mappe...');
  LFiles := TDirectory.GetFiles(Config.MapPath);
  for LFileName in LFiles do
  begin
    TAction.ExecuteFile(LFileName);
  end;
  TLogger.Log(llInfo, 'Mappe caricate.');
end;

procedure TfrmMain.Log(ALogLevel: TLogLevel; const AMessage: string);
const
  LevelColor: array [TLogLevel] of TColor = (clBlue, clGreen, clOlive, clRed);
begin
  memLog.SelAttributes.Color := LevelColor[ALogLevel];
  memLog.SelText := DateTimeToStr(Now) + ': ' + AMessage + sLineBreak;

  // Forza il richedit a posizionarsi sull'ultima riga
  if Visible then
    memLog.SetFocus;
  memLog.SelStart := memLog.GetTextLen;
  memLog.Perform(EM_SCROLLCARET, 0, 0);

  Application.ProcessMessages;
end;

procedure TfrmMain.lstScriptsClick(Sender: TObject);
var
  LFileName: string;
begin
  if lstScripts.ItemIndex >= 0 then
  begin
    LFileName := TPath.Combine(Config.ScriptPath, lstScripts.Items[lstScripts.ItemIndex]);
    memScript.Text := TFile.ReadAllText(LFileName);
  end;

end;

end.
