unit ScriptTest.UI.MainForm;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants,
  System.Classes, System.Types, System.IOUtils, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.StdCtrls,
  DBBuilder.Core.Logging, Vcl.ComCtrls;

type
  TfrmScriptTest = class(TForm, ILogger)
    memSource: TMemo;
    memLog: TRichEdit;
    btnRun: TButton;
    cmbFileName: TComboBox;
    btnLoad: TButton;
    btnClearState: TButton;
    Button1: TButton;
    procedure btnClearStateClick(Sender: TObject);
    procedure btnLoadClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure btnRunClick(Sender: TObject);
    procedure cmbFileNameCloseUp(Sender: TObject);
    procedure memSourceKeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure Button1Click(Sender: TObject);
  private
    procedure InitFileList;
    procedure LoadFile(const AFileName: string);
  public
    procedure Log(ALogLevel: TLogLevel; const AMessage: string);
  end;

var
  frmScriptTest: TfrmScriptTest;

implementation

{$R *.dfm}

uses
  DBBuilder.Core.Lexer, DBBuilder.ISCScript.Interpreter, DBBuilder.Core.Config,
  DBBuilder.ISCScript.Processor;

procedure TfrmScriptTest.btnClearStateClick(Sender: TObject);
begin
  DefaultScriptProcessor.ClearMaps;
end;

procedure TfrmScriptTest.btnLoadClick(Sender: TObject);
begin
  LoadFile(cmbFileName.Text);
end;

procedure TfrmScriptTest.LoadFile(const AFileName: string);
begin
  memSource.Lines.LoadFromFile(AFileName);
end;

procedure TfrmScriptTest.FormDestroy(Sender: TObject);
begin
  RegisterLogger(nil);
end;

procedure TfrmScriptTest.InitFileList;
var
  LTestDir: string;
  LFiles: TStringDynArray;
  LFileName: string;
begin
  LTestDir := TPath.Combine(Config.AppPath, 'tests');
  ForceDirectories(LTestDir);
  LFiles := TDirectory.GetFiles(LTestDir);
  cmbFileName.Clear;
  for LFileName in LFiles do
    cmbFileName.Items.Add(LFileName);
end;

procedure TfrmScriptTest.FormCreate(Sender: TObject);
begin
  RegisterLogger(Self);

  InitFileList;
end;

procedure TfrmScriptTest.btnRunClick(Sender: TObject);
var
  LRunner: TRunner;
begin
  LRunner := TRunner.CreateFromString(memSource.Text, DefaultScriptProcessor);
  try
    LRunner.Execute;
  finally
    LRunner.Free;
  end;
end;

procedure TfrmScriptTest.Button1Click(Sender: TObject);
begin
  memLog.Clear;
end;

procedure TfrmScriptTest.cmbFileNameCloseUp(Sender: TObject);
begin
  if cmbFileName.ItemIndex >= 0 then
    LoadFile(cmbFileName.Items[cmbFileName.ItemIndex]);
end;

procedure TfrmScriptTest.Log(ALogLevel: TLogLevel; const AMessage: string);
const
  LevelColor: array [TLogLevel] of TColor = (clBlue, clGreen, clOlive, clRed);
begin
  memLog.SelAttributes.Color := LevelColor[ALogLevel];
  memLog.SelText := DateTimeToStr(Now) + ': ' + AMessage + sLineBreak;

  // Forza il richedit a posizionarsi sull'ultima riga
  if Visible then
    memLog.SetFocus;
  memLog.SelStart := memLog.GetTextLen;
  memLog.Perform(EM_SCROLLCARET, 0, 0);

  Application.ProcessMessages;

  if Visible then
    memSource.SetFocus;
end;

procedure TfrmScriptTest.memSourceKeyUp(Sender: TObject; var Key: Word; Shift:
    TShiftState);
begin
  if Key = VK_F9 then
    btnRun.Click;
end;

end.
